#include <cassert>
#include <iostream>

#include "board.h"


// CONSTRUCTOR
Board::Board(unsigned int w, unsigned int h) {
  height_ = h;
  width_ = w;
  board = std::vector<std::vector<char> >(h,std::vector<char>(w,' '));
}

// ACCESSOR
char Board::get(unsigned int y, unsigned int x) const { 
  assert (y >= 0 && y < board.size());
  assert (x >= 0 && x < board[y].size());
  return board[y][x]; 
}

// helper function for set
char IntToChar(unsigned int v) {
  assert (v >= 1 && v <= 30);
  if (v <= 9) {
    return char('0' + v);
  } else {
    // A = 10
    // B = 11
    // C = 12
    // etc
    return char('A' + (v-10));
  }
}

// MODIFIER
void Board::set(unsigned int x, unsigned int y, unsigned int v) {
  assert (board.size() == height_);
  assert (board[0].size() == width_);
  assert (y >= 0 && y < board.size());
  assert (x >= 0 && x < board[y].size());
  assert(board[y][x] == ' ');
  board[y][x] = IntToChar(v);
}

// PRINT FUNCTION
std::ostream& operator<<(std::ostream& ostr, const Board &b) {
  // top frame
  ostr << "+" << std::string(b.width(),'-') << "+" << std::endl;
  for (int y = b.height()-1; y >= 0; y--) {
    ostr << "|";
    // each row
    for (unsigned int x = 0; x < b.width(); x++) {
      ostr << b.get(y,x);
    }
    ostr << "|" << std::endl;
  }
  // bottom frame
  ostr << "+" << std::string(b.width(),'-') << "+" << std::endl;
  return ostr;
}
